
%%                                                        STAGtimedat  7.01

%                                                  plots StagYY's time.dat
%    . calls f_designBackground
%    . calls f_defaultsTimedat
%    . calls f_dimParameter
%    . calls f_designFigure
%    . calls f_importTimedat
%    . calls f_saveFigure
%    . calls f_varia
%                                               11.07.2016  Fabio Crameri

%% UPDATES 7.0
% *more stable design-routines*
% *cleaner plot design and layout*
% *automatic figure size*

%% UPDATES 6.0
% *improved plot layout*
% *improved code design*
% *bug fixes to dimensionalisation*

%% NOTES

function [SAVE] = STAGtimedat(IN,PLOT,SWITCH,STYLE,SAVE)
if exist('IN','var') %if run from a parfile
    %% DEFAULT VARIABLES
    %...see f_defaultsTimedat.m
else
    clear; clf;
    %% INPUT (...use parameter file instead)
    SAVE.StartNum = 1; SAVE.EndNum = 1;
    [IN,PLOT,SWITCH,STYLE,SAVE] = f_defaultsTimedat; %get default variables
    IN.name         =   {  'testDIM'  'testDIM' };	% filename
    IN.parameter    =   [   11        	];
    IN.folder       =   {   '/work/plotTest/'   };
end

%% STARTUP PROCEDURE
SAVE.app = 'STAGtimedat'; SAVE.appVersion = 7.01;
if ~isfield(SAVE,'count'); SAVE.count = 0; end
[fAIo,SAVE,SWITCH] = f_startup(PLOT,SAVE,SWITCH);
if ~strcmp(fAIo.status,'all fine!'); disp(' '); disp(['                         ',char(10004),' Finished.']); return; end

%% VARIABLE ADJUSTMENTS
nrModels = size(IN.name,2);
% check for length of number input arrays
if size(IN.parameter,2)<nrModels
    dummy = ones(1,nrModels-size(IN.parameter,2)); dummy(:) = IN.parameter(1,1);
    IN.parameter = [IN.parameter, dummy];
end
% check for length of string input arrays
if size(IN.folder,2)<nrModels
    dummy = cell(1,nrModels-size(IN.folder,2)); dummy(:) = IN.folder(1,1);
    IN.folder = [IN.folder, dummy];
end

%% SWITCHES ADJUSTMENT
if SWITCH.simplify && PLOT.Performance %Plots with different xlabel
    SWITCH.simplify = false;
    if SWITCH.verbose; disp('SWITCH.simplify has been turned off due to Performance plot.'); end 
end

%% GET NUMBER OF SUBPLOTS
nrPlots=0;                                                               % timedat label
if PLOT.Temperature;        nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'Tmean'}; end
if PLOT.Viscosity;      	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'eta_mean'}; end
if PLOT.Velocity;           nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'Vrms'}; end
if PLOT.Ra_eff;             nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'ra_eff'}; end
if PLOT.Nu_top;             nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'Nu_top'}; end
if PLOT.Nu_bot;             nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'Nu_bot'}; end
if PLOT.InternalHeating;	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'H_int'}; end
if PLOT.Composition;        nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'C_mean'}; end
if PLOT.HeatfluxTop;    	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'F_bot'}; end
if PLOT.HeatfluxBot;    	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'F_top'}; end
if PLOT.EruptionRate;   	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'erupt_rate'}; end
if PLOT.EruptHeatFlux;   	nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'erupt_heatflux'}; end
if PLOT.Erupta;             nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'erupta'}; end
if PLOT.Entrainment;        nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'entrainment'}; end
if PLOT.Cmass_error;        nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'Cmass_error'}; end
if PLOT.Timesteps;          nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'istep'}; end
if PLOT.Performance;        nrPlots = nrPlots+1; DATA.string(1,nrPlots)={'istep'}; end

%% LAYOUT SUBPLOTS
if nrPlots<=4
    nxSubplots  = 1;
    nzSubplots  = nrPlots;
else
    nxSubplots  = min(nrPlots,2);
    nzSubplots  = floor((nrPlots+1)/2);
end

%% SETUP FIGURE
clf(figure(1))
%color mode
if strcmp(STYLE.colorMode,'light')
    STYLE.colorModeBW   = 'white';
elseif strcmp(STYLE.colorMode,'dark')
    STYLE.colorModeBW   = 'black';
end
set(gcf,'color',STYLE.colorModeBW);
% set figure position
FPOS.x          = 47;
FPOS.y          = 1;
% set figure width and height
if nzSubplots==1 && nxSubplots==1;          FPOS.size = [717 255];	%[1 1]
elseif nzSubplots==2 && nxSubplots==1;    	FPOS.size = [478 330];	%[2 1]
elseif nzSubplots==3 && nxSubplots==1;      FPOS.size = [478 490];	%[3 1]
elseif nzSubplots==4 && nxSubplots==1;      FPOS.size = [478 655];	%[4 1]
elseif nzSubplots==1 && nxSubplots==2;    	FPOS.size = [956 170];	%[1 2]
elseif nzSubplots==3 && nxSubplots==2;      FPOS.size = [956 490];	%[3 2]
elseif nzSubplots==4 && nxSubplots==2;      FPOS.size = [956 655];	%[4 2]
elseif nzSubplots==5 && nxSubplots==2;      FPOS.size = [956 820];	%[5 2]
elseif nzSubplots==6 && nxSubplots==2;      FPOS.size = [956 980];	%[6 2]
else
    FPOS.size   = [956 980]; %dummy
end
FPOS.position   = [FPOS.x FPOS.y FPOS.size(1,1) FPOS.size(1,2)];
set(gcf,'Position',FPOS.position);    

h_runs = zeros(size(IN.name,2),1);
for irun=1:size(IN.name,2)
    clearvars -except IN PLOT SWITCH STYLE SAVE DATA irun h_runs nrPlots nxSubplots nzSubplots
    
    %% VARIABLE CONVERSION & SETUP
    FILE.name         	= IN.name{irun};
    folder              = IN.folder{irun};
    SWITCH.parameter    = IN.parameter(irun);
    lineColor           = PLOT.ColorVector(irun,:);
    
    %% CHECK FOR DIRECTORY
    folder_op       = strcat(folder,'+op/');
    folder_im       = strcat(folder,'+im/');
    FILE.directory  = [folder_op,FILE.name,'/']; %needed for f_dimParameter
    filestem        = strcat(folder_op,FILE.name,'/',FILE.name,'_time.dat');
    if ~exist(filestem,'file')
        disp(filestem)
        error('file not found: CHECK FILESTEM!')
    end
    
    %% OPEN FILE and check its length
    %find number of header entries in first line automatically
    data1 = fopen(filestem,'r');
    nHeaders=0;
    entry=1;
    while (entry ~= '0')
        entry = fscanf(data1,'%s',[1 1]);
        nHeaders = nHeaders+1;
        DATA.title{1,nHeaders} = entry;
    end
    fclose(data1);
    DATA.title(:,end) = [];
    nHeaders=nHeaders-1;
    
    if IN.checkFileSize
        %find number of numeric entries in one line automatically
        data1 = fopen(filestem,'r');
        header1 = fgets(data1); %skip header
        nHeaders=0;
        entry=0;
        while (nHeaders<=n_headers || entry~=1)  %there could be an entry~=1 somewhere else the timestep...
            entry = fscanf(data1,'%e',[1 1]);
            nHeaders = nHeaders+1;
        end
        fclose(data1);
        nHeaders=nHeaders-1;
        n = nHeaders;
        
        %find line number automatically
        data1 = fopen(filestem,'r');
        nLines = 0;
        while (fgets(data1) ~= -1)
            nLines = nLines+1;
        end
        fclose(data1);
        nr_time_steps = nLines;  % #data in z-direction
        
        disp(['datafile is ',num2str(n),' x ',num2str(nLines)])
    end
    
    %% IMPORT & READING DATA
    if nHeaders==27 || nHeaders==17 %new or old files
        DATA.filename = filestem;
        [DATA] = f_importTimedat(DATA);
    else
        error('number of columns in time.dat is different from standard format!')
    end
    nColumns    = size(DATA.Array,2);
    nRows       = size(DATA.Array{1},1);
    
    %% DISPLAY MODEL DETAILS I
    disp(['Name:              ',FILE.name])
    if SWITCH.verbose
        % disp(['datafile is ',num2str(n),' x ',num2str(nLines)])
        disp(['Data Structure:       __',num2str(nColumns),'__>'])
        disp('                     |')
        disp(['                   ',num2str(nRows)])
        disp('                     |')
        disp('                     V')
    end
    
    %% CHECK FOR DIMENSIONAL INPUT:
    %get time after first timestep
    dummy = DATA.Array{strcmp(DATA.title,'time')}(2,1);
    if dummy>1e2  %if first timestep > 100 (years) then DIMENSIONAL INPUT
        SWITCH.dimensionalInput = true; dimString = 'dimensional';
        SWITCH.dim = true; %set output to dimensional if input is dimensional
    else
        SWITCH.dimensionalInput = false; dimString = 'non-dimensional';
    end
    disp(['Data:              ',SWITCH.GeodynamicCode,', ',dimString])
    
    %% DIMENSIONALIZATION
    [SETUP,SWITCH] = f_dimParameter(SWITCH,FILE);
    
    % SET TIME
    timeRow          	= find(strcmp(DATA.title,'time'));
    PLOT.time         	= DATA.Array{1,timeRow};
    PLOT.time_dim     	= DATA.Array{1,timeRow}*SETUP.timescale/SETUP.secyear;  %get dimensional time [a]
    
    dummy = max(PLOT.time_dim); %[a]
    if dummy>100*1e6 %>100Ma
        PLOT.time2plot = PLOT.time_dim/1e9; PLOT.time2plotDim = 'Ga'; PLOT.timeConvert = SETUP.timescale/SETUP.secyear/1e9;
    elseif dummy<1e-3 %<0.001a
        PLOT.time2plot = PLOT.time_dim*SETUP.secyear; PLOT.time2plotDim = 's'; PLOT.timeConvert = SETUP.timescale;
    elseif dummy<1e2 %<100a
        PLOT.time2plot = PLOT.time_dim; PLOT.time2plotDim = 'a'; PLOT.timeConvert = SETUP.timescale/SETUP.secyear;
    elseif dummy<1e5 %<0.1Ma
        PLOT.time2plot = PLOT.time_dim/1e3; PLOT.time2plotDim = 'ka'; PLOT.timeConvert = SETUP.timescale/SETUP.secyear/1e3;
    else
        PLOT.time2plot = PLOT.time_dim/1e6; PLOT.time2plotDim = 'Ma'; PLOT.timeConvert = SETUP.timescale/SETUP.secyear/1e6;
    end
    if ~SWITCH.dim && ~SWITCH.dimensionalInput; PLOT.time2plot = PLOT.time; PLOT.time2plotDim = 'nd'; PLOT.timeConvert = 1; end
    
    DATA.Array{1,timeRow}       = PLOT.time2plot; %dimensionalise time array
    
    minTime                     = min(PLOT.time); %[nd or s]
    maxTime                     = max(PLOT.time); %[nd or s]
    minTimeDim                  = minTime*PLOT.timeConvert; %[whatever]
    maxTimeDim                  = maxTime*PLOT.timeConvert; %[whatever]
    
    %  1       2   3     4      5   6      7   8    9    10   11        12      13      14      15   16
    %  istep time F_top F_bot Tmin Tmean Tmax Vmin Vrms Vmax eta_min eta_mean eta_max ra_eff Nu_top Nu_bot
    
    %  17       18   19     20    21      22        23      24              25          26       27
    %  C_min C_mean C_max F_mean F_max erupt_rate erupta erupt_heatflux entrainment Cmass_error H_int
    
    %% SET UP FIELD VARIABLES
    %  	1                 	2                   3                   4                   5                   6
    %  	data name           variable label      rprof.dat label    	dim. scale      	data dimension      individual range
    FIELD_M = {
        'Time Steps'        'cumul. time steps' 'istep'          	1                   '#'                 false           %1
        'Time'              'time'              'time'           	PLOT.timeConvert    PLOT.time2plotDim   false           %2
        'Top Heatflux'     	'F_{top}'           'F_top'             SETUP.HFscale      	SETUP.HFDim       	false           %3
        'Bottom Heatflux'  	'F_{bot}'           'F_bot'             SETUP.HFscale    	SETUP.HFDim       	false           %4
        'Min. Temperature'  'T_{min}'           'Tmin'              SETUP.Tscale        SETUP.TDim       	false           %5
        'Temperature'       'T_{mean}'          'Tmean'             SETUP.Tscale        SETUP.TDim       	true            %6
        'Max. Temperature'  'T_{max}'           'Tmax'              SETUP.Tscale        SETUP.TDim       	false           %7
        'Min. Velocity'     'v_{min}'           'vmin'              SETUP.Vscale        SETUP.vDim       	false           %8
        'Velocity'          'v_{mean}'        	'vrms'              SETUP.Vscale        SETUP.vDim       	true            %9
        'Max. Velocity'     'v_{max}'           'vmax'              SETUP.Vscale        SETUP.vDim       	false           %10
        'Min. Viscosity'    '\eta_{min}'        'eta_min'         	SETUP.etascale    	SETUP.etaDim     	false           %11
        'Viscosity'         '\eta_{mean}'       'eta_mean'        	SETUP.etascale     	SETUP.etaDim     	true            %12
        'Max. Viscosity'    '\eta_{max}'        'eta_max'         	SETUP.etascale    	SETUP.etaDim      	false           %13
        'Ra_{eff}'          'Ra_{eff}'          'ra_eff'           	1                   ''                  false           %14
        'Nu_{top}'          'Nu_{top}'          'Nu_top'          	1                   ''                  false           %15
        'Nu_{bot}'          'Nu_{bot}'          'Nu_bot'           	1                   ''                  false           %16
        'Min. Composition' 	'C_{min}'           'C_min'             1                   ''                  false           %17
        'Composition'     	'C_{mean}'        	'C_mean'          	1                   ''                  true            %18
        'Max. Composition' 	'C_{max}'          	'C_max'             1                   ''                  false           %19
        'Mean Melt Fraction' 'F_{mean}'       	'F_mean'           	1                   ''                  false           %20
        'Max. Melt Fraction' 'F_{max}'       	'F_max'             1                   ''                  false           %21
        'Eruption Rate'     'UCvarepsilondot'  	'erupt_rate'      	SETUP.Vscale       	SETUP.vDim       	false           %22
        'Erupta'            'erupta'            'erupta'          	SETUP.lengthscale 	SETUP.lengthDim    	false           %23
        'Eruption Heatflux' 'hf_{erupt}'        'erupt_heatflux'   	SETUP.HFscale    	SETUP.HFDim       	false           %24
        'Entrainment'       'en'                'entrainment'      	1                   ''                  false           %25
        'Conserv. Error'    'Cmass_{err}'       'Cmass_error'     	1                   ''                  false           %26
        'Internal Heating'  'H_{int}'           'H_int'          	SETUP.Hscale        SETUP.HDim       	false           %27
        };
    
    % UNICODE VARIABLES
    %use the following character sequence to add unicode variables:
    %UCvarepsilondot => char(941)
    %UCvarepsilon => char(949)
    FIELD_M(:,2) = strrep(FIELD_M(:,2),'UCvarepsilondot',{char(941)}); %does only work if string consists ONLY of unicode character
    FIELD_M(:,2) = strrep(FIELD_M(:,2),'UCvarepsilon',{char(949)}); %does only work if string consists ONLY of unicode character
    
    % FIELD VARIABLES ADJUSTMENTS
    if SWITCH.dimensionalInput
        DATA.Array{1,14} = DATA.Array{1,14}.*SETUP.eta0.*SETUP.Ra0; %undo Ra_eff = Ra_0/eta_mean if dimensional mode
    end
    if SWITCH.dim || SWITCH.dimensionalInput
        %nothing to do
    else
        %adjust for non-dimensional parameters
        FIELD_M(:,4) = {1};
        FIELD_M(:,5) = {'nd'};
    end
    
    %% DISPLAY MODEL DETAILS II
    disp(['Top-BC:            ',SETUP.topBC])
    disp(['Time Range:        ',num2str(minTimeDim,3),' - ',num2str(maxTimeDim,3),' ',PLOT.time2plotDim])
    
    %% COLOURING ADJUSTMENTS
    if strcmp(STYLE.colorMode,'dark'); lineColor = 1-lineColor; end
    
    %% PLOTTING
    BottomSubplot = zeros(nrPlots,1);
    num_subplot=0;
    for iPlot=1:nrPlots
        %set parameter to plot
        numRow   = find(strcmp(DATA.title,DATA.string(iPlot)));
        
        FIELD.name          = FIELD_M{numRow,1};
        FIELD.symbol        = FIELD_M{numRow,2};
        FIELD.dim           = FIELD_M{numRow,5};
        FIELD.logarithmic   = false; %dummy needed for f_varia
        
        %dimensionalise data
        DATA.Array{1,numRow} = DATA.Array{1,numRow}*FIELD_M{numRow,4};
        if SWITCH.parameterRange && FIELD_M{numRow,6} %parameter range
            DATA.Array{1,numRow-1} = DATA.Array{1,numRow-1}*FIELD_M{numRow-1,4};
            DATA.Array{1,numRow+1} = DATA.Array{1,numRow+1}*FIELD_M{numRow+1,4};
        end
        
        %set subplot
        if nrPlots>1
            num_subplot	= num_subplot+1;
            hax         = subplot(nzSubplots,nxSubplots,num_subplot);
        end
        if iPlot==nrPlots || iPlot==nrPlots-1
            BottomSubplot(iPlot,1) = true;
        end
        hold on
        
        %performance plot
        if PLOT.Performance && iPlot==nrPlots
            dt = zeros(size(DATA.Array{1,timeRow},1),1);
            for i=1:size(DATA.Array{1,timeRow},1)
                if i==size(DATA.Array{1,timeRow},1)
                    dt(i,1) = DATA.Array{1,timeRow}(i,:)-DATA.Array{1,timeRow}(i-1,:);
                else
                    dt(i,1) = DATA.Array{1,timeRow}(i+1,:)-DATA.Array{1,timeRow}(i,:);
                end
            end
            pp1 = plot(DATA.Array{numRow}, dt,'Color',lineColor);
            
            VARIA.xlabelName    = FIELD.name;
            VARIA.xlabelDim     = FIELD.dim;
            VARIA.zlabelName    = 'Time Step';
            VARIA.zlabelDim     = PLOT.time2plotDim;
            VARIA.task          = 'create annotation strings';
            [PLOT,VARIA] = f_varia(VARIA,FIELD,[],SWITCH,PLOT,STYLE,SAVE);
            xlabel(PLOT.xlabel)
            ylabel(PLOT.ylabel)
            
            grid on
            if SWITCH.MinMaxLegend
                legend('^{dt}/_{time step}')
            end
            if iPlot==nrPlots
                h_runs(irun,1) = pp1;
            end
            break
        end
        
        %time plot
        if SWITCH.parameterRange && FIELD_M{numRow,6}
            colourMax   = [0.64 0.08 0.18];
            colourMin   = [0.0 0.45 0.74];
            if PLOT.rangeAsArea
                %min-max area
                edgecolor = 'none';
                areaAlpha = 0.2;
                hold on
                maxData     = DATA.Array{numRow+1};
                minData     = DATA.Array{numRow-1};
                xData       = DATA.Array{1,timeRow};
                pp4 = fill([xData' fliplr(xData')],[maxData' fliplr(minData')],...
                    lineColor,'EdgeColor',edgecolor,'EdgeAlpha',areaAlpha,'FaceAlpha',areaAlpha);
            else
                %min/max lines
                hold on
                pp2 = plot(DATA.Array{1,timeRow}, DATA.Array{numRow-1},'Color',colourMin); 	%minimum
                pp3 = plot(DATA.Array{1,timeRow}, DATA.Array{numRow+1},'Color',colourMax);	%maximum
            end
            pp1 = plot(DATA.Array{1,timeRow}, DATA.Array{numRow},'Color',lineColor);   %mean

            if SWITCH.MinMaxLegend
                if PLOT.rangeAsArea
                    legend([pp1,pp4],[FIELD_M(numRow,2),strcat(FIELD_M(numRow-1,2),'-',FIELD_M(numRow+1,2))])
                else
                    legend([pp3,pp1,pp2],[FIELD_M(numRow+1,2),FIELD_M(numRow,2),FIELD_M(numRow-1,2)],'Location','Best');
                end
            end
        else
            pp1 = plot(DATA.Array{1,timeRow}, DATA.Array{numRow},'Color',lineColor);   %mean
            if SWITCH.MinMaxLegend
                legend(pp1,FIELD_M(numRow,2),'Location','Best');
            end
        end
        if iPlot==nrPlots
            h_runs(irun,1) = pp1;
        end
        
        %axes labelling
%         if SWITCH.dim || SWITCH.dimensionalInput
%             if strcmp(FIELD_M(timeRow,5),'-') %xlabel
%                 dimStringTime 	= '';
%             else
%                 dimStringTime 	= FIELD_M{timeRow,5};
%             end
%         else
%             dimStringTime   = 'nd';
%         end
        dimStringTime 	= FIELD_M{timeRow,5};

        VARIA.xlabelName    = FIELD_M{timeRow,1};
        VARIA.xlabelDim     = dimStringTime;
        VARIA.zlabelName    = FIELD.name;
        VARIA.zlabelDim     = FIELD.dim;
        VARIA.task          = 'create annotation strings';
        [PLOT,VARIA] = f_varia(VARIA,FIELD,[],SWITCH,PLOT,STYLE,SAVE);
        if SWITCH.simplify && ~BottomSubplot(iPlot,1)
            %no xlabel
        else
            xlabel(PLOT.xlabel); 
        end
        ylabel(PLOT.ylabel)
        
        grid on
    end %loop
    IN.saveString = ([IN.saveString,FILE.name,'_']);
    disp(' ')
end

%% LEGEND
if ~SWITCH.MinMaxLegend %not applied if individual legends are switched on
    hl = legend(h_runs, IN.name, 'Location', 'NorthEast', 'interpreter', 'none');
    % hl = legend(h_runs, IN.name, 'Location', 'SouthEast');
end

%% STYLE PLOT
if SWITCH.stylePlots
    %     STYLE.Legend = false; %done later
    if SWITCH.styleLegend; STYLE.Legend = true; else; STYLE.Legend = false; end
    f_designFigure(STYLE);
end
if SWITCH.styleBackground
    BACK.ApplyTo    = 'all';          %'current', 'all'
    BACK.colorMode  = STYLE.colorMode;
    f_designBackground(BACK) %set plot background of each subplot
end

%% ANNOTATION
if SWITCH.annotation
    %     pause(0.5);
    f_designAnnotations(PLOT,STYLE);
end

%% SAVING PLOTS
if SAVE.Plot
    SAVE.Directory = strcat(folder_im,FILE.name,'/');
    if strcmp(STYLE.colorMode,'light')
        SAVE.blackBackground=0;
    elseif strcmp(STYLE.colorMode,'dark')
        SAVE.blackBackground=1;
    end
    if strcmp(STYLE.colorMode,'dark'); endix='_dark'; else; endix=''; end
    %     SaveDirectory = [file_stem_im,fname,'/'];
    SAVE.FigureName = strcat(IN.saveString,endix);   %file name
    if SWITCH.dim
        SAVE.FigureName = strcat(SAVE.FigureName,'_DIM');
    end
    SAVE.FigureNr               = 1;
    SAVE.keepAspect             = true;
    SAVE.jpgResolution       	= '-r500';
    
    f_saveFigure( SAVE )
end

%% FINISHING UP
disp(' ')
disp(['                         ',char(10004),' Finished.']);










